"""Version information for cybersec-scanner."""

__version__ = "0.1.0"
__version_info__ = tuple(int(i) for i in __version__.split("."))

__title__ = "cybersec-scanner"
__description__ = "A comprehensive security scanner and RAG-based vulnerability analyzer"
__author__ = "CyberSec Team"
__license__ = "MIT"
